--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

-- Purpose:
-- Description:

LIBRARY IEEE,common_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_barrel_shift IS
  GENERIC (
    g_nof_streams : NATURAL;
    g_data_w      : NATURAL
  );
  PORT (
    rst         : IN  STD_LOGIC;
    clk         : IN  STD_LOGIC;

    snk_in_arr  : IN  t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
    shift_in    : IN STD_LOGIC_VECTOR(ceil_log2(g_nof_streams)-1 DOWNTO 0);

    src_out_arr : OUT t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
    shift_out   : IN STD_LOGIC_VECTOR(ceil_log2(g_nof_streams)-1 DOWNTO 0)
  );
END dp_barrel_shift;


ARCHITECTURE str OF dp_barrel_shift IS

  TYPE t_reg IS RECORD
    src_out_arr : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
    shift_out   : STD_LOGIC_VECTOR(ceil_log2(g_nof_streams)-1 DOWNTO 0);
  END RECORD;

  CONSTANT c_reg_defaults : t_reg := ( (OTHERS=>c_dp_sosi_rst),
                                       (OTHERS=>'0') );
 
  SIGNAL r, nxt_r : t_reg := c_reg_defaults;

BEGIN
   
  r <= nxt_r WHEN rising_edge(clk);

  p_comb : PROCESS(rst, r, snk_in_arr, shift_in)
    VARIABLE v : t_reg;
  BEGIN
    v := r;

    FOR i IN 0 TO g_nof_streams-1 LOOP
      v.src_out_arr(i+TO_UINT(shift_in)):=snk_in_arr(i);
    END LOOP;

    IF rst = '1' THEN
      v := c_reg_defaults;
    END IF;
 
    nxt_r <= v;
  END PROCESS;
 
END str;
