-------------------------------------------------------------------------------
--
-- Copyright (C) 2010
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;

-- Purpose:
--   The extended eop output can be used to gate valid eop related stream info
--   like empty and error.
-- Description:
--   Extend the eop until the next sop.

ENTITY dp_eop_extend IS
  PORT (
    rst        : IN  STD_LOGIC;
    clk        : IN  STD_LOGIC;
    in_sop     : IN  STD_LOGIC;
    in_eop     : IN  STD_LOGIC;
    eop_extend : OUT STD_LOGIC
  );
END dp_eop_extend;


ARCHITECTURE rtl OF dp_eop_extend IS

  SIGNAL extend     : STD_LOGIC;

BEGIN

  u_extend : ENTITY common_lib.common_switch
  PORT MAP (
    rst         => rst,
    clk         => clk,
    switch_high => in_eop,
    switch_low  => in_sop,
    out_level   => extend
  );  
                
  eop_extend <= (in_eop OR extend) AND NOT in_sop;

END rtl;
