-------------------------------------------------------------------------------
--
-- Copyright (C) 2017
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Author:
-- . Daniel van der Schuur
-- Purpose: 
-- . Provide monitoring and control of dp_fifo_*
-- Description:
-- . Instantiate this dp_fifo_monitor next to dp_fifo_* and connect the
--   desired monitoring inputs and control outputs.

LIBRARY IEEE, common_lib, mm_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_fifo_monitor IS
  PORT (
    mm_clk       : IN  STD_LOGIC;
    mm_rst       : IN  STD_LOGIC;

    dp_clk       : IN  STD_LOGIC;
    dp_rst       : IN  STD_LOGIC;

    reg_mosi     : IN  t_mem_mosi;
    reg_miso     : OUT t_mem_miso;
    -- Status inputs
    rd_usedw_32b : IN  STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0) := (OTHERS=>'0');
    wr_usedw_32b : IN  STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0) := (OTHERS=>'0');
    rd_emp       : IN  STD_LOGIC := '0';
    wr_full      : IN  STD_LOGIC := '0';
    -- Control output
    rd_fill_32b  : OUT STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0)
  );
END dp_fifo_monitor;


ARCHITECTURE str OF dp_fifo_monitor IS

  CONSTANT c_field_arr : t_common_field_arr(4 DOWNTO 0) := ( (field_name_pad("rd_usedw"), "RO", 32, field_default(0) ),
                                                             (field_name_pad("wr_usedw"), "RO", 32, field_default(0) ),
                                                             (field_name_pad("rd_empty"), "RO",  1, field_default(0) ),
                                                             (field_name_pad("wr_full" ), "RO",  1, field_default(0) ),
                                                             (field_name_pad("rd_fill" ), "RW", 32, field_default(0) ));

  SIGNAL mm_fields_in  : STD_LOGIC_VECTOR(field_slv_in_len(c_field_arr)-1 DOWNTO 0);
  SIGNAL mm_fields_out : STD_LOGIC_VECTOR(field_slv_out_len(c_field_arr)-1 DOWNTO 0);
      
BEGIN

  mm_fields_in(field_hi(c_field_arr, "rd_usedw") DOWNTO field_lo(c_field_arr, "rd_usedw")) <= rd_usedw_32b;
  mm_fields_in(field_hi(c_field_arr, "wr_usedw") DOWNTO field_lo(c_field_arr, "wr_usedw")) <= wr_usedw_32b;
  mm_fields_in(field_hi(c_field_arr, "rd_empty") DOWNTO field_lo(c_field_arr, "rd_empty")) <= slv(rd_emp);
  mm_fields_in(field_hi(c_field_arr, "wr_full")  DOWNTO field_lo(c_field_arr, "wr_full"))  <= slv(wr_full);
     
  u_mm_fields: ENTITY mm_lib.mm_fields
  GENERIC MAP(
    g_use_slv_in_val  => FALSE,    -- use FALSE to save logic when always slv_in_val='1'
    g_field_arr       => c_field_arr
  )
  PORT MAP (
    mm_clk     => mm_clk,
    mm_rst     => mm_rst,

    mm_mosi    => reg_mosi,
    mm_miso    => reg_miso,
    
    slv_clk    => dp_clk,
    slv_rst    => dp_rst, 

    slv_in     => mm_fields_in,
    slv_in_val => '1',

    slv_out    => mm_fields_out
  );

  rd_fill_32b <= mm_fields_out(field_hi(c_field_arr, "rd_fill") DOWNTO field_lo(c_field_arr, "rd_fill"));
      
END str;

