-------------------------------------------------------------------------------
--
-- Copyright (C) 2017
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Author:
-- . Daniel van der Schuur
-- Purpose: 
-- . Array of dp_fifo_monitor instances
-- Description:
-- . see dp_fifo_monitor.vhd

LIBRARY IEEE, common_lib, mm_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_fifo_monitor_arr IS
  GENERIC (
    g_nof_streams : NATURAL
  );
  PORT (
    mm_clk           : IN  STD_LOGIC;
    mm_rst           : IN  STD_LOGIC;

    dp_clk           : IN  STD_LOGIC;
    dp_rst           : IN  STD_LOGIC;

    reg_mosi         : IN  t_mem_mosi;
    reg_miso         : OUT t_mem_miso;
    -- Status inputs
    rd_usedw_32b_arr : IN  t_slv_32_arr(g_nof_streams-1 DOWNTO 0) := (OTHERS=>(OTHERS=>'0'));
    wr_usedw_32b_arr : IN  t_slv_32_arr(g_nof_streams-1 DOWNTO 0) := (OTHERS=>(OTHERS=>'0')); 
    rd_emp_arr       : IN  STD_LOGIC_VECTOR(g_nof_streams-1 DOWNTO 0) := (OTHERS=>'0');
    wr_full_arr      : IN  STD_LOGIC_VECTOR(g_nof_streams-1 DOWNTO 0) := (OTHERS=>'0');
    -- Control output
    rd_fill_32b_arr  : OUT t_slv_32_arr(g_nof_streams-1 DOWNTO 0)
  );
END dp_fifo_monitor_arr;


ARCHITECTURE str OF dp_fifo_monitor_arr IS

  CONSTANT c_nof_regs : NATURAL := 5;

  SIGNAL reg_mosi_arr : t_mem_mosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL reg_miso_arr : t_mem_miso_arr(g_nof_streams-1 DOWNTO 0); 
     
BEGIN

  u_common_mem_mux : ENTITY common_lib.common_mem_mux
  GENERIC MAP (    
    g_nof_mosi    => g_nof_streams,
    g_mult_addr_w => ceil_log2(c_nof_regs)
  )
  PORT MAP (
    mosi     => reg_mosi,
    miso     => reg_miso,
    mosi_arr => reg_mosi_arr,
    miso_arr => reg_miso_arr
  );

  gen_dp_fifo_monitor : FOR i IN 0 TO g_nof_streams-1 GENERATE
    u_dp_fifo_monitor: ENTITY work.dp_fifo_monitor
    PORT MAP (
      mm_clk       => mm_clk,
      mm_rst       => mm_rst,

      dp_clk       => dp_clk,
      dp_rst       => dp_rst,
  
      reg_mosi     => reg_mosi_arr(i),
      reg_miso     => reg_miso_arr(i),

      rd_usedw_32b => rd_usedw_32b_arr(i),
      wr_usedw_32b => wr_usedw_32b_arr(i), 
      rd_emp       => rd_emp_arr(i),       
      wr_full      => wr_full_arr(i),      
      rd_fill_32b  => rd_fill_32b_arr(i)
    );     
  END GENERATE;

END str;

