-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;

ENTITY dp_fifo_to_mm_reg IS
  PORT (
    -- Clocks and reset
    mm_rst            : IN  STD_LOGIC;  -- reset synchronous with mm_clk
    mm_clk            : IN  STD_LOGIC;  -- memory-mapped bus clock
      
    -- Memory Mapped Slave
    sla_in            : IN  t_mem_mosi; -- actual ranges defined by c_mm_reg
    sla_out           : OUT t_mem_miso; -- actual ranges defined by c_mm_reg
    
    -- MM registers 
    mm_rd_usedw       : IN  STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0)

   );
END dp_fifo_to_mm_reg;


ARCHITECTURE rtl OF dp_fifo_to_mm_reg IS

  CONSTANT c_mm_reg        : t_c_mem := (latency  => 1,
                                         adr_w    => ceil_log2(1),
                                         dat_w    => c_word_w,       -- Use MM bus data width = c_word_w = 32 for all MM registers
                                         nof_dat  => 1,
                                         init_sl  => '0');                                               
     
BEGIN

  p_mm_reg : PROCESS (mm_rst, mm_clk)
  BEGIN
    IF mm_rst = '1' THEN
      -- Read access
      sla_out   <= c_mem_miso_rst;

    ELSIF rising_edge(mm_clk) THEN
      -- Read access defaults
      sla_out.rdval <= '0';
      
      -- Read access: get register value
      IF sla_in.rd = '1' THEN
        sla_out       <= c_mem_miso_rst;    -- set unused rddata bits to '0' when read
        sla_out.rdval <= '1';               -- c_mm_reg.latency = 1
        CASE TO_UINT(sla_in.address(c_mm_reg.adr_w-1 DOWNTO 0)) IS
          -- Read Block Sync
          WHEN 0 =>            
            sla_out.rddata(c_word_w-1 DOWNTO 0) <= mm_rd_usedw;
          WHEN OTHERS => NULL;  -- unused MM addresses
        END CASE;
      END IF;
    END IF;
  END PROCESS;

END rtl;

