--------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_frame_remove IS
  GENERIC (
    g_data_w          : NATURAL;
    g_symbol_w        : NATURAL;
    g_hdr_nof_words   : NATURAL;
    g_tail_nof_words  : NATURAL;
    g_snk_latency     : NATURAL := 1;
    g_internal_bypass : BOOLEAN := FALSE -- can be used to eliminate external GENERATE-statements/wiring
  );
  PORT (
    mm_rst  : IN  STD_LOGIC;
    mm_clk  : IN  STD_LOGIC;

    st_rst  : IN  STD_LOGIC;
    st_clk  : IN  STD_LOGIC;

    snk_out : OUT t_dp_siso;
    snk_in  : IN  t_dp_sosi;

    sla_in  : IN  t_mem_mosi := c_mem_mosi_rst;
    sla_out : OUT t_mem_miso; 

    src_in  : IN  t_dp_siso;
    src_out : OUT t_dp_sosi
  );
END dp_frame_remove;


ARCHITECTURE str OF dp_frame_remove IS

  CONSTANT c_tail_nof_symbols : NATURAL := (g_tail_nof_words*g_symbol_w)/g_symbol_w;

  SIGNAL hdr_rem_siso   : t_dp_siso;
  SIGNAL hdr_rem_sosi   : t_dp_sosi;

  SIGNAL snk_out_rl_1   : t_dp_siso;
  SIGNAL snk_in_rl_1    : t_dp_sosi;

BEGIN

  no_bypass : IF g_internal_bypass = FALSE GENERATE

    u_dp_latency_adpapter: ENTITY work.dp_latency_adapter
    GENERIC MAP (
      g_in_latency  => g_snk_latency,
      g_out_latency => 1
    )
    PORT MAP (
      rst      => st_rst,
      clk      => st_clk,
     
      snk_out  => snk_out,
      snk_in   => snk_in,
  
      src_out  => snk_in_rl_1,
      src_in   => snk_out_rl_1       
    ); 

    u_dp_hdr_remove : ENTITY work.dp_hdr_remove
    GENERIC MAP (            
      g_data_w        => g_data_w,
      g_symbol_w      => g_symbol_w,
      g_hdr_nof_words => g_hdr_nof_words
    )                        
    PORT MAP (               
      mm_rst      => mm_rst, 
      mm_clk      => mm_clk, 
                             
      st_rst      => st_rst, 
      st_clk      => st_clk, 
                             
      snk_out     => snk_out_rl_1,
      snk_in      => snk_in_rl_1,
  
      sla_in      => sla_in,
      sla_out     => sla_out,
  
      src_in      => hdr_rem_siso,
      src_out     => hdr_rem_sosi
    );

    u_dp_tail_remove : ENTITY work.dp_tail_remove
    GENERIC MAP (
      g_data_w      => g_data_w,
      g_symbol_w    => g_symbol_w,
      g_nof_symbols => c_tail_nof_symbols 
    )
    PORT MAP (
      st_rst  => st_rst,
      st_clk  => st_clk,
   
      snk_out => hdr_rem_siso, 
      snk_in  => hdr_rem_sosi,
  
      src_in  => src_in,   
      src_out => src_out 
    );

  END GENERATE;

  gen_bypass : IF g_internal_bypass = TRUE GENERATE
    src_out <= snk_in;
    snk_out <= src_in;
  END GENERATE;  
  
END str;
