--------------------------------------------------------------------------------
--
-- Copyright (C) 2010
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------
 
-- Reuse from LOFAR rad_frame_repack.vhd and rad_frame_repack(rtl).vhd

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;


ENTITY dp_frame_repack IS
  GENERIC (
    g_in_dat_w        : NATURAL;
    g_in_nof_words    : NATURAL;
    g_out_dat_w       : NATURAL;
    g_out_nof_words   : NATURAL
  );
  PORT (
    rst              : IN  STD_LOGIC;
    clk              : IN  STD_LOGIC;
    in_dat           : IN  STD_LOGIC_VECTOR(g_in_dat_w-1 DOWNTO 0);
    in_val           : IN  STD_LOGIC;
    in_sof           : IN  STD_LOGIC;
    in_eof           : IN  STD_LOGIC;
    out_dat          : OUT STD_LOGIC_VECTOR(g_out_dat_w-1 DOWNTO 0);
    out_val          : OUT STD_LOGIC;
    out_sof          : OUT STD_LOGIC;
    out_eof          : OUT STD_LOGIC
  );
END dp_frame_repack;


ARCHITECTURE str OF dp_frame_repack IS

  SIGNAL pack_fsn       : STD_LOGIC_VECTOR(in_dat'RANGE);
  SIGNAL pack_dat       : STD_LOGIC_VECTOR(in_dat'RANGE);
  SIGNAL pack_val       : STD_LOGIC;
  SIGNAL pack_sof       : STD_LOGIC;
  SIGNAL pack_eof       : STD_LOGIC;
  SIGNAL pack_err       : STD_LOGIC;

  SIGNAL repack_fsn     : STD_LOGIC_VECTOR(out_dat'RANGE);
  SIGNAL repack_dat     : STD_LOGIC_VECTOR(out_dat'RANGE);
  SIGNAL repack_val     : STD_LOGIC;
  SIGNAL repack_sof     : STD_LOGIC;
  SIGNAL repack_eof     : STD_LOGIC;

BEGIN

  no_pack : IF g_in_nof_words=g_out_nof_words GENERATE
    out_dat <= RESIZE_UVEC(in_dat, out_dat'LENGTH);  -- any extra bits will get stripped again by dp_repack at the other end,
                                                     -- typically g_out_dat_w=g_in_dat_w
    out_val <= in_val;
    out_sof <= in_sof;
    out_eof <= in_eof;
  END GENERATE;
  
  gen_pack : IF g_in_nof_words/=g_out_nof_words GENERATE
    unframe : ENTITY work.dp_unframe
    GENERIC MAP (
      g_dat_w        => g_in_dat_w,    
      g_fsn_w        => g_in_dat_w
    )
    PORT MAP (
      rst            => rst,
      clk            => clk,
      in_dat         => in_dat,
      in_val         => in_val,
      in_sof         => in_sof,
      in_eof         => in_eof,
      out_fsn        => pack_fsn,
      out_sync       => OPEN,
      out_dat        => pack_dat,
      out_val        => pack_val,
      out_sof        => pack_sof,
      out_eof        => pack_eof,
      out_err        => pack_err
    );
    
    repack : ENTITY work.dp_repack_legacy
    GENERIC MAP (
      g_in_dat_w      => g_in_dat_w,
      g_in_nof_words  => g_in_nof_words,
      g_out_dat_w     => g_out_dat_w,
      g_out_nof_words => g_out_nof_words
    )
    PORT MAP (
      rst            => rst,
      clk            => clk,
      in_dat         => pack_dat,
      in_val         => pack_val,
      in_sof         => pack_sof,
      in_eof         => pack_eof,
      out_dat        => repack_dat,
      out_val        => repack_val,
      out_sof        => repack_sof,
      out_eof        => repack_eof
    );
    
    repack_fsn <= RESIZE_SVEC(pack_fsn,repack_fsn'LENGTH);  -- pack_fsn remains valid
    
    frame : ENTITY work.dp_frame
    GENERIC MAP (
      g_dat_w        => g_out_dat_w,
      g_fsn_w        => g_out_dat_w
    )
    PORT MAP (
      rst            => rst,
      clk            => clk,
      in_fsn         => repack_fsn,
      in_dat         => repack_dat,
      in_val         => repack_val,
      in_sof         => repack_sof,
      in_eof         => repack_eof,
      in_err         => pack_err,                           -- pack_err remains valid
      out_dat        => out_dat,
      out_val        => out_val,
      out_sof        => out_sof,
      out_eof        => out_eof
    );
  END GENERATE;
  
END str;
