--------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_hdr_remove IS
  GENERIC (
    g_data_w        : NATURAL;
    g_symbol_w      : NATURAL;
    g_hdr_nof_words : NATURAL
  );
  PORT (
    mm_rst  : IN  STD_LOGIC;
    mm_clk  : IN  STD_LOGIC;

    st_rst  : IN  STD_LOGIC;
    st_clk  : IN  STD_LOGIC;

    snk_out : OUT t_dp_siso;
    snk_in  : IN  t_dp_sosi;

    sla_in   : IN  t_mem_mosi; 
    sla_out  : OUT t_mem_miso; 

    src_in  : IN  t_dp_siso;
    src_out : OUT t_dp_sosi
  );
END dp_hdr_remove;


ARCHITECTURE str OF dp_hdr_remove IS

  CONSTANT c_nof_symbols : NATURAL := (g_data_w * g_hdr_nof_words) / g_symbol_w;

  -- element (0) gets the head, element (1) gets the tail
  SIGNAL split_siso_arr : t_dp_siso_arr(0 TO 1);
  SIGNAL split_sosi_arr : t_dp_sosi_arr(0 TO 1);
  
BEGIN

  u_dp_ram_to_mm : ENTITY work.dp_ram_to_mm
  GENERIC MAP (
    g_ram_rd_nof_words => g_hdr_nof_words * (g_data_w/c_word_w),
    g_ram_wr_dat_w     => g_data_w 
  )
  PORT MAP (
    mm_rst           => mm_rst,
    mm_clk           => mm_clk,

    st_rst           => st_rst,
    st_clk           => st_clk,

    sla_out          => sla_out,
    sla_in           => sla_in,

    snk_in           => split_sosi_arr(0),
    snk_out          => split_siso_arr(0)
  );

  split_siso_arr(1)  <= src_in;
  src_out            <= split_sosi_arr(1);

  u_split : ENTITY work.dp_split  -- RL = 1
  GENERIC MAP (
    g_data_w        => g_data_w,
    g_symbol_w      => g_symbol_w,
    g_nof_symbols   => c_nof_symbols
  )
  PORT MAP (
    rst         => st_rst,
    clk         => st_clk,
    -- ST sinks
    snk_out     => snk_out,
    snk_in      => snk_in,
    -- ST source
    src_in_arr  => split_siso_arr,
    src_out_arr => split_sosi_arr
  );
  
END str;
