-------------------------------------------------------------------------------
--
-- Copyright (C) 2010
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.all;

-- Purpose:
--   Hold the in_data value when in_en is inactive. The held data can then be
--   output when the downstream sink is ready again.
-- Description:
--   Hold last in_data value when not valid else register the new in_data.
-- Remarks:
-- . The rst can be left not connected, because typically the data value at
--   reset is don't care (because the data valid will then be low)
-- . The in_en is typically in_valid or ready
-- . The in_data is typically in_data, in_empty, in_error or in_channel
-- . Typically used together with dp_hold_ctrl

ENTITY dp_hold_data IS
  PORT (
    rst      : IN  STD_LOGIC := '0';
    clk      : IN  STD_LOGIC;
    in_data  : IN  STD_LOGIC_VECTOR;
    in_en    : IN  STD_LOGIC;
    hld_data : OUT STD_LOGIC_VECTOR
  );
END dp_hold_data;


ARCHITECTURE rtl OF dp_hold_data IS
  
  SIGNAL i_hld_data   : STD_LOGIC_VECTOR(hld_data'RANGE);
  SIGNAL nxt_hld_data : STD_LOGIC_VECTOR(hld_data'RANGE);

BEGIN

  hld_data <= i_hld_data;

  nxt_hld_data <= in_data WHEN in_en='1' ELSE i_hld_data;
  
  p_clk : PROCESS(rst, clk)
  BEGIN
    IF rst='1' THEN
      i_hld_data <= (OTHERS=>'0');
    ELSIF rising_edge(clk) THEN
      i_hld_data <= nxt_hld_data;
    END IF;
  END PROCESS;

END rtl;
