-------------------------------------------------------------------------------
--
-- Copyright (C) 2011
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- The dp_mon is dropped inbetween a source and a sink. It does not affect the
-- passing stream in any way - it monitors the stream and keeps useful stats.

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.std_logic_1164.all;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

ENTITY dp_mon IS
  GENERIC (
    g_latency : NATURAL := 1
  );
  PORT (
    rst          : IN  STD_LOGIC := '0';
    clk          : IN  STD_LOGIC;
    en           : IN  STD_LOGIC;

    snk_out      : OUT t_dp_siso;
    snk_in       : IN  t_dp_sosi;

    src_in       : IN  t_dp_siso;
    src_out      : OUT t_dp_sosi;

    clr          : IN  STD_LOGIC := '0'; -- clears the word count

    -- All output stats could be in a record (t_dp_mon) in the future
    word_cnt     : OUT STD_LOGIC_VECTOR(32-1 DOWNTO 0) -- Number of words accepted by the connected sink
  );
END dp_mon;


ARCHITECTURE rtl OF dp_mon IS

  SIGNAL snk_accept      : STD_LOGIC;

BEGIN

  snk_out <= src_in;
  src_out <= snk_in;

  snk_accept <= snk_in.valid WHEN g_latency > 0 ELSE snk_in.valid AND src_in.ready;
  
  u_word_cntr : ENTITY common_lib.common_counter
  GENERIC MAP (
    g_width     => 32,
    g_step_size => 1,
    g_latency   => 0
  )
  PORT MAP (
    rst     => rst,
    clk     => clk,
    cnt_clr => clr,
    cnt_en  => snk_accept,
    count   => word_cnt
  );

END rtl;

