-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

ENTITY dp_offload_rx_legacy IS
  GENERIC (
    g_nof_streams             : NATURAL := 1;
    g_data_w                  : NATURAL;
    g_hdr_nof_words           : NATURAL;
    g_remove_crc              : BOOLEAN;
    g_crc_nof_words           : NATURAL
  ); 
  PORT (
    mm_rst                   : IN  STD_LOGIC;
    mm_clk                   : IN  STD_LOGIC; 
    
    st_rst                   : IN  STD_LOGIC;
    st_clk                   : IN  STD_LOGIC;  

    -- MM registers: hdr_remove
    ram_hdr_remove_mosi      : IN  t_mem_mosi;
    ram_hdr_remove_miso      : OUT t_mem_miso;

    dp_sosi_arr              : OUT t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0); 
    dp_siso_arr              : IN  t_dp_siso_arr(g_nof_streams-1 DOWNTO 0);              
                                                                          
    rx_sosi_arr              : IN  t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0); 
    rx_siso_arr              : OUT t_dp_siso_arr(g_nof_streams-1 DOWNTO 0)                             
  );
END dp_offload_rx_legacy;


ARCHITECTURE str OF dp_offload_rx_legacy IS 

  CONSTANT c_hdr_remove_ram_addr_w : NATURAL := ceil_log2( g_hdr_nof_words * (g_data_w/c_word_w) );

  SIGNAL rx_pkt_sosi_arr         : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL rx_pkt_siso_arr         : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0);

  SIGNAL ram_hdr_remove_mosi_arr : t_mem_mosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL ram_hdr_remove_miso_arr : t_mem_miso_arr(g_nof_streams-1 DOWNTO 0);

BEGIN

  u_common_mem_mux_hdr_ram : ENTITY common_lib.common_mem_mux
  GENERIC MAP (    
    g_nof_mosi    => g_nof_streams,
    g_mult_addr_w => c_hdr_remove_ram_addr_w
  )
  PORT MAP (
    mosi     => ram_hdr_remove_mosi,
    miso     => ram_hdr_remove_miso,
    mosi_arr => ram_hdr_remove_mosi_arr,
    miso_arr => ram_hdr_remove_miso_arr
  );

  gen_nof_streams: FOR i IN 0 TO g_nof_streams-1 GENERATE

    ---------------------------------------------------------------------------------------
    -- RX: Unframe: remove header (and CRC if phy link is used) from DP packets
    ---------------------------------------------------------------------------------------           
    u_frame_remove : ENTITY dp_lib.dp_frame_remove
    GENERIC MAP (            
      g_data_w         => g_data_w,
      g_symbol_w       => c_byte_w,
      g_hdr_nof_words  => g_hdr_nof_words,
      g_tail_nof_words => sel_a_b(g_remove_crc, g_crc_nof_words, 0)
    )                        
    PORT MAP (               
      mm_rst      => mm_rst, 
      mm_clk      => mm_clk, 
                             
      st_rst      => st_rst, 
      st_clk      => st_clk, 
                             
      snk_out     => rx_siso_arr(i),
      snk_in      => rx_sosi_arr(i),
  
      -- dp_frame_remove uses hdr_remove internally
      sla_in      => ram_hdr_remove_mosi_arr(i),
      sla_out     => ram_hdr_remove_miso_arr(i),
  
      src_in      => rx_pkt_siso_arr(i),
      src_out     => rx_pkt_sosi_arr(i)
    );
  
    ---------------------------------------------------------------------------------------
    -- RX: Convert DP packets to DP stream
    ---------------------------------------------------------------------------------------      
    u_dp_packet_dec : ENTITY dp_lib.dp_packet_dec
    GENERIC MAP (
      g_data_w => g_data_w
    )
    PORT MAP (
      rst       => st_rst,
      clk       => st_clk,
  
      snk_out   => rx_pkt_siso_arr(i),
      snk_in    => rx_pkt_sosi_arr(i),
  
      src_in    => dp_siso_arr(i),
      src_out   => dp_sosi_arr(i)  
    );

  END GENERATE;

END str;
