--------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE,common_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE work.dp_stream_pkg.ALL;

-- Purpose:
--   Assert output during a packet on the input:
--
--                 ___     _________     ___
-- val      ______|   |___|         |___|   |____         
--                 ___                    
-- sop      ______|   |__________________________
--                                       ___                          
-- eop      ____________________________|   |____           
--
--                 _________________________
-- pkt_det  ______|                         |____


ENTITY dp_packet_detect IS
  GENERIC (
    g_latency   : NATURAL := 1
  );
  PORT (
    rst         : IN  STD_LOGIC;
    clk         : IN  STD_LOGIC;

    -- The SOSI/SISO buses of the stream we're probing
    sosi        : IN  t_dp_sosi;
    siso        : IN  t_dp_siso := c_dp_siso_rst; -- Connect this when RL=0

    pkt_det     : OUT STD_LOGIC
  );
END dp_packet_detect;

ARCHITECTURE str OF dp_packet_detect IS

  SIGNAL i_pkt_det    : STD_LOGIC;
  SIGNAL prev_pkt_det : STD_LOGIC;
  SIGNAL prev_eop     : STD_LOGIC;

  -- For RL=0, these signals are only asserted when src_in.ready='1'.
  SIGNAL sop          : STD_LOGIC;
  SIGNAL eop          : STD_LOGIC;

BEGIN

  gen_rl_0: IF g_latency=0 GENERATE
    sop <= sosi.sop AND siso.ready;
    eop <= sosi.eop AND siso.ready;
  END GENERATE;

  gen_wires: IF g_latency>0 GENERATE
    sop <= sosi.sop;
    eop <= sosi.eop;
  END GENERATE;

  pkt_det <= i_pkt_det;
 
  p_snk : PROCESS(sop, eop, prev_pkt_det, prev_eop)
  BEGIN
    i_pkt_det <= prev_pkt_det;
    IF prev_eop = '1' THEN
      i_pkt_det <= '0';
    END IF;
    IF sop = '1' THEN
      i_pkt_det <= '1';
    END IF;
  END PROCESS;
   
  p_reg : PROCESS(rst, clk)
  BEGIN
    IF rst='1' THEN
      prev_eop <= '0';
      prev_pkt_det <= '0';
    ELSIF rising_edge(clk) THEN
      prev_eop <= eop;
      prev_pkt_det <= i_pkt_det;
    END IF;
  END PROCESS;
  
END str;
