--------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

-- Purpose: Unmerge each input packet into g_nof_pkt output packets. Below is the 
--          waveform when g_nof_pkt = 3. The numbers in snk_in.sop and snk_in.eop 
--          match pkt_cnt.
--                    _                             _                             _
-- snk_in.sop   _____|0|___________________________|1|___________________________|2|_
--                                                _                             _    
-- snk_in.sop   _________________________________|0|___________________________|1|___
--                    _         _         _         _         _         _         _       
-- src_out.sop  _____|0|_______|1|_______|2|_______|0|_______|1|_______|2|_______|0|_
--                            _         _         _         _         _         _       
-- src_out.eop  _____________|0|_______|1|_______|2|_______|0|_______|1|_______|2|___


LIBRARY IEEE,common_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_packet_unmerge IS
  GENERIC (
    g_nof_pkt   : NATURAL := 1; -- Nof packets to unmerge each incoming packet to
    g_pkt_len   : NATURAL := 1  -- Length of the unmerged packets 
  ); 
  PORT (
    rst         : IN  STD_LOGIC;
    clk         : IN  STD_LOGIC;

    snk_out     : OUT t_dp_siso;
    snk_in      : IN  t_dp_sosi;

    src_in      : IN  t_dp_siso;
    src_out     : OUT t_dp_sosi
  );
END dp_packet_unmerge;


ARCHITECTURE rtl OF dp_packet_unmerge IS

BEGIN

  -- Temporary void component that assigns output = input
  snk_out <= src_in;
  src_out <= snk_in;

END rtl;
