-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;

ENTITY dp_ram_from_mm_reg IS
  GENERIC (
    g_dp_on_at_init : STD_LOGIC := '0' -- Enable data path at default
  );
  PORT (
    mm_rst            : IN  STD_LOGIC;
    mm_clk            : IN  STD_LOGIC;

    st_rst            : IN  STD_LOGIC;
    st_clk            : IN  STD_LOGIC;
      
    sla_in            : IN  t_mem_mosi;
    
    dp_on             : OUT STD_LOGIC
   );
END dp_ram_from_mm_reg;


ARCHITECTURE rtl OF dp_ram_from_mm_reg IS

  CONSTANT c_mm_reg  : t_c_mem := (latency  => 1,
                                   adr_w    => ceil_log2(1),
                                   dat_w    => c_word_w,
                                   nof_dat  => 1,
                                   init_sl  => '0');

  SIGNAL mm_dp_on : STD_LOGIC;                                 
     
BEGIN

  p_mm_reg : PROCESS (mm_rst, mm_clk)
  BEGIN
    IF mm_rst = '1' THEN
      mm_dp_on <= g_dp_on_at_init;
    ELSIF rising_edge(mm_clk) THEN
      -- Write access: set register value
      IF sla_in.wr = '1' THEN
        CASE TO_UINT(sla_in.address(c_mm_reg.adr_w-1 DOWNTO 0)) IS
          -- Write Block Sync
          WHEN 0 =>            
            mm_dp_on <= sla_in.wrdata(0);
          WHEN OTHERS => NULL;  -- unused MM addresses
        END CASE;
      END IF;
    END IF;
  END PROCESS;

  u_async_dp_on : ENTITY common_lib.common_async
  GENERIC MAP (
    g_rst_level => '0'
  )
  PORT MAP (
    rst  => st_rst,
    clk  => st_clk,
    din  => mm_dp_on,
    dout => dp_on
  );

END rtl;

