-------------------------------------------------------------------------------
--
-- Copyright (C) 2010
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE work.dp_stream_pkg.ALL;

-- Purpose:
-- =======
-- This component can be added to an uninterruptible source to provide the
-- stream with some flow control based on the ready signal of the
-- connected sink.

ENTITY dp_ready IS
  GENERIC (
    g_ready_latency : NATURAL := 1  -- >= 0
  );
  PORT (
    rst          : IN  STD_LOGIC;
    clk          : IN  STD_LOGIC;
    -- ST sink
    snk_in       : IN  t_dp_sosi;
    -- ST source
    src_in       : IN  t_dp_siso;
    src_out      : OUT t_dp_sosi
  );
END dp_ready;


ARCHITECTURE rtl OF dp_ready IS
  
  SIGNAL reg_ready : STD_LOGIC_VECTOR(g_ready_latency DOWNTO 0);
  SIGNAL reg_val   : STD_LOGIC;
    
BEGIN

  gen_latency : IF g_ready_latency>0 GENERATE    
    reg_ready(0) <= src_in.ready;  -- use reg_ready(0) to combinatorially store src_in.ready
    
    p_clk : PROCESS(rst, clk)
    BEGIN
      IF rst='1' THEN
        reg_ready(g_ready_latency DOWNTO 1) <= (OTHERS=>'0');
      ELSIF rising_edge(clk) THEN
        reg_ready(g_ready_latency DOWNTO 1) <= reg_ready(g_ready_latency-1 DOWNTO 0);
      END IF;
    END PROCESS;
    
    reg_val <= reg_ready(g_ready_latency);
  END GENERATE gen_latency;

  no_latency : IF g_ready_latency=0 GENERATE
    reg_val <= src_in.ready;     
  END GENERATE;

  p_src_out : PROCESS(snk_in, reg_val)
  BEGIN
    src_out       <= snk_in;
    src_out.sync  <= snk_in.sync  AND reg_val;
    src_out.valid <= snk_in.valid AND reg_val;
    src_out.sop   <= snk_in.sop   AND reg_val;
    src_out.eop   <= snk_in.eop   AND reg_val;
  END PROCESS;
  
END rtl;

