-------------------------------------------------------------------------------
--
-- Copyright 2020
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--
-- Author: R. van der Walle
-- Purpose: Single instance wrapper for dp_selector_arr.
-- Description:
-- Remark:
-- . See dp_selector_arr.vhd for more detail.
-------------------------------------------------------------------------------


LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
LIBRARY common_lib, common_mult_lib;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_selector IS
  GENERIC (
    g_pipeline    : NATURAL := 1
  );
  PORT (
    mm_rst                  : IN  STD_LOGIC;
    mm_clk                  : IN  STD_LOGIC;
    dp_rst                  : IN  STD_LOGIC := '0';
    dp_clk                  : IN  STD_LOGIC;

    -- MM interface
    reg_selector_mosi       : IN  t_mem_mosi := c_mem_mosi_rst; 
    reg_selector_miso       : OUT t_mem_miso;

    pipe_sosi               : IN  t_dp_sosi;
    ref_sosi                : IN  t_dp_sosi;
    out_sosi                : OUT t_dp_sosi
  );
END dp_selector;

ARCHITECTURE str OF dp_selector IS

BEGIN

  u_dp_selector_arr : ENTITY work.dp_selector_arr
  GENERIC MAP (
    g_nof_arr   => 1,
    g_pipeline  => g_pipeline
  )
  PORT MAP (
    mm_rst             => mm_rst,
    mm_clk             => mm_clk,
    dp_rst             => dp_rst,
    dp_clk             => dp_clk,

    reg_selector_mosi  => reg_selector_mosi,
    reg_selector_miso  => reg_selector_miso,

    pipe_sosi_arr(0)   =>  pipe_sosi,  
    ref_sosi_arr(0)    =>  ref_sosi,
    out_sosi_arr(0)    =>  out_sosi
  );

END str;
