-------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose:
-- . Throttle snk_out.ready to provide a parallel sink with data blocks at a
--   consistent, regular interval
-- Description:
-- . g_period is 

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE work.dp_stream_pkg.ALL;

ENTITY dp_throttle_sop IS
  GENERIC (
    g_period : NATURAL
  );
  PORT (
    rst      : IN  STD_LOGIC;
    clk      : IN  STD_LOGIC;

    snk_out  : OUT t_dp_siso;
    snk_in   : IN  t_dp_sosi 
  );
END dp_throttle_sop;

ARCHITECTURE str OF dp_throttle_sop IS

  CONSTANT c_cnt_w : NATURAL := ceil_log2(g_period);

  SIGNAL cnt_clr   : STD_LOGIC := '0'; 
  SIGNAL cnt_en    : STD_LOGIC := '0'; 
  SIGNAL cycle_cnt : STD_LOGIC_VECTOR(c_cnt_w-1 DOWNTO 0);

  SIGNAL switch_high : STD_LOGIC;
  SIGNAL switch_low  : STD_LOGIC;
  SIGNAL switch_out  : STD_LOGIC;

BEGIN

  snk_out.xon <= '1';

  u_common_counter : ENTITY common_lib.common_counter
  GENERIC MAP (
    g_width     => c_cnt_w
  )
  PORT MAP (
    rst     => rst,
    clk     => clk,
    cnt_clr => cnt_clr,
    cnt_en  => cnt_en,
    count   => cycle_cnt
  );

  cnt_en  <= '1' WHEN TO_UINT(cycle_cnt) < g_period-2 ELSE '0';
  cnt_clr <= snk_in.sop;

  u_common_switch : ENTITY common_lib.common_switch
  GENERIC MAP (
    g_rst_level    => '1', 
    g_priority_lo  => FALSE,
    g_or_high      => TRUE,
    g_and_low      => TRUE
  )
  PORT MAP (
    clk         => clk,
    rst         => rst,
    switch_high => switch_high,
    switch_low  => switch_low,
    out_level   => switch_out
  );

  switch_high   <= NOT cnt_en;
  switch_low    <= snk_in.eop;
  snk_out.ready <= switch_out;

END str;

