-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE work.dp_stream_pkg.ALL;

-- Purpose : MMS for dp_bsn_scheduler
-- Description: See dp_bsn_scheduler.vhd

ENTITY mms_dp_bsn_scheduler IS
  GENERIC (
    g_cross_clock_domain : BOOLEAN := TRUE;  -- use FALSE when mm_clk and dp_clk are the same, else use TRUE to cross the clock domain
    g_bsn_w              : NATURAL := c_dp_stream_bsn_w
  );
  PORT (
    -- Memory-mapped clock domain
    mm_rst      : IN  STD_LOGIC;
    mm_clk      : IN  STD_LOGIC;
    
    reg_mosi    : IN  t_mem_mosi;
    reg_miso    : OUT t_mem_miso;
    
    -- Streaming clock domain
    dp_rst      : IN  STD_LOGIC;
    dp_clk      : IN  STD_LOGIC;
    
    snk_in      : IN  t_dp_sosi;  -- only uses eop (= block sync), bsn[]
    trigger_out : OUT STD_LOGIC
  );
END mms_dp_bsn_scheduler;


ARCHITECTURE str OF mms_dp_bsn_scheduler IS

  SIGNAL scheduled_bsn : STD_LOGIC_VECTOR(g_bsn_w-1 DOWNTO 0) := (OTHERS=>'0');
  
BEGIN

  u_mm_reg : ENTITY work.dp_bsn_scheduler_reg
  GENERIC MAP (
    g_cross_clock_domain => g_cross_clock_domain
  )
  PORT MAP (
    -- Clocks and reset
    mm_rst            => mm_rst,
    mm_clk            => mm_clk,
    st_rst            => dp_rst,
    st_clk            => dp_clk,
    
    -- Memory Mapped Slave in mm_clk domain
    sla_in            => reg_mosi,
    sla_out           => reg_miso,
    
    -- MM registers in dp_clk domain
    st_current_bsn    => snk_in.bsn,
    st_scheduled_bsn  => scheduled_bsn
  );
  
  u_bsn_scheduler : ENTITY work.dp_bsn_scheduler
  GENERIC MAP (
    g_bsn_w       => g_bsn_w
  )
  PORT MAP (
    rst           => dp_rst,
    clk           => dp_clk,
    -- MM control
    scheduled_bsn => scheduled_bsn,
    -- Streaming
    snk_in        => snk_in,
    trigger_out   => trigger_out
  );
  
END str;

