-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

ENTITY mms_dp_fifo_from_mm IS
  GENERIC (
    g_sim           : BOOLEAN := FALSE; 
    g_wr_fifo_depth : NATURAL 
  );
  PORT (
    mm_rst          : IN    STD_LOGIC;
    mm_clk          : IN    STD_LOGIC;
 
    wr_sosi         : OUT   t_dp_sosi;
    
    -- MM registers
    ctrl_mosi       : IN    t_mem_mosi := c_mem_mosi_rst;
    ctrl_miso       : OUT   t_mem_miso;
  
    data_mosi       : IN    t_mem_mosi := c_mem_mosi_rst;
    data_miso       : OUT   t_mem_miso := c_mem_miso_rst;

    wr_usedw        : IN    STD_LOGIC_VECTOR(ceil_log2(g_wr_fifo_depth)-1 DOWNTO 0)
  );
END mms_dp_fifo_from_mm;


ARCHITECTURE str OF mms_dp_fifo_from_mm IS
 
  SIGNAL mm_wr_usedw       : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL mm_wr_availw      : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL mm_wr_data        : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL mm_wr             : STD_LOGIC;

BEGIN

  u_dp_fifo_from_mm : ENTITY dp_lib.dp_fifo_from_mm
  GENERIC MAP(
    g_fifo_size => g_wr_fifo_depth
  )
  PORT MAP (
     rst        => mm_rst,
     clk        => mm_clk,
     
     src_out    => wr_sosi,
     usedw      => wr_usedw, -- used words from rd FIFO
     
     mm_wr      => mm_wr,
     mm_wrdata  => mm_wr_data,
     mm_usedw   => mm_wr_usedw, --resized to 32 bits
     mm_availw  => mm_wr_availw  --resized to 32 bits
  );

  u_dp_fifo_from_mm_reg: ENTITY work.dp_fifo_from_mm_reg            
  PORT MAP (       
    mm_rst            => mm_rst, 
    mm_clk            => mm_clk,
    
    sla_in            => ctrl_mosi,
    sla_out           => ctrl_miso,
    
    mm_wr_usedw       => mm_wr_usedw,
    mm_wr_availw      => mm_wr_availw
  );

  mm_wr_data <= data_mosi.wrdata(c_word_w-1 DOWNTO 0); 
  mm_wr      <= data_mosi.wr;
 
END str;

