-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

ENTITY mms_dp_fifo_to_mm IS
  GENERIC (
    g_sim           : BOOLEAN := FALSE; 
    g_rd_fifo_depth : NATURAL 
  );
  PORT (
    mm_rst          : IN    STD_LOGIC;
    mm_clk          : IN    STD_LOGIC;
 
    rd_sosi         : IN    t_dp_sosi;
    rd_siso         : OUT   t_dp_siso;
    
    -- MM registers
    ctrl_mosi       : IN    t_mem_mosi := c_mem_mosi_rst;
    ctrl_miso       : OUT   t_mem_miso;
  
    data_mosi       : IN    t_mem_mosi := c_mem_mosi_rst;
    data_miso       : OUT   t_mem_miso := c_mem_miso_rst;

    rd_usedw        : IN    STD_LOGIC_VECTOR(ceil_log2(g_rd_fifo_depth)-1 DOWNTO 0)

  );
END mms_dp_fifo_to_mm;


ARCHITECTURE str OF mms_dp_fifo_to_mm IS
 
  SIGNAL mm_rd_usedw       : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL mm_rd_data        : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL mm_rd_val         : STD_LOGIC;
  SIGNAL mm_rd             : STD_LOGIC;

BEGIN

  u_dp_fifo_to_mm : ENTITY dp_lib.dp_fifo_to_mm
  GENERIC MAP(
    g_fifo_size => g_rd_fifo_depth
  )
  PORT MAP (
     rst       => mm_rst,
     clk       => mm_clk,
     
     snk_out   => rd_siso,
     snk_in    => rd_sosi,
     usedw     => rd_usedw, -- used words from rd FIFO
     
     mm_rd     => mm_rd,
     mm_rddata => mm_rd_data,
     mm_rdval  => mm_rd_val,
     mm_usedw  => mm_rd_usedw --resized to 32 bits
  );

  u_dp_fifo_to_mm_reg: ENTITY work.dp_fifo_to_mm_reg            
  PORT MAP (       
    mm_rst            => mm_rst, 
    mm_clk            => mm_clk,
    
    sla_in            => ctrl_mosi,
    sla_out           => ctrl_miso,
    
    mm_rd_usedw       => mm_rd_usedw
  );

  data_miso.rddata(c_word_w-1 DOWNTO 0) <= mm_rd_data; 
  data_miso.rdval                       <= mm_rd_val; 
  mm_rd                                 <= data_mosi.rd;
 
END str;

