-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

ENTITY mms_dp_ram_from_mm IS
  GENERIC (
    g_ram_wr_nof_words : NATURAL;
    g_ram_rd_dat_w     : NATURAL;
    g_init_file        : STRING := "UNUSED";
    g_dp_on_at_init    : STD_LOGIC := '0' -- Enable data path at default
  );
  PORT (
    mm_rst    : IN  STD_LOGIC;
    mm_clk    : IN  STD_LOGIC;

    st_rst    : IN  STD_LOGIC;
    st_clk    : IN  STD_LOGIC;

    reg_mosi  : IN  t_mem_mosi := c_mem_mosi_rst;   
    ram_mosi  : IN  t_mem_mosi := c_mem_mosi_rst;

    src_in    : IN  t_dp_siso;
    src_out   : OUT t_dp_sosi
  );
END mms_dp_ram_from_mm;


ARCHITECTURE str OF mms_dp_ram_from_mm IS
 
 SIGNAL dp_on  : STD_LOGIC;

BEGIN

 u_dp_ram_from_mm : ENTITY dp_lib.dp_ram_from_mm
  GENERIC MAP(
    g_ram_wr_nof_words => g_ram_wr_nof_words, 
    g_ram_rd_dat_w     => g_ram_rd_dat_w,     
    g_init_file        => g_init_file
  )
  PORT MAP (
    mm_rst    => mm_rst,
    mm_clk    => mm_clk,

    st_rst    => st_rst,
    st_clk    => st_clk,

    mm_addr   => ram_mosi.address,
    mm_wr     => ram_mosi.wr,
    mm_wrdata => ram_mosi.wrdata,

    dp_on     => dp_on,

    src_in    => src_in,
    src_out   => src_out
  );

  u_dp_ram_from_mm_reg: ENTITY work.dp_ram_from_mm_reg   
  GENERIC MAP(
    g_dp_on_at_init    => g_dp_on_at_init
  )
  PORT MAP (       
    mm_rst  => mm_rst, 
    mm_clk  => mm_clk,

    st_rst  => st_rst,
    st_clk  => st_clk,
    
    sla_in  => reg_mosi,
    
    dp_on   => dp_on 
  );

END str;
