-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE work.dp_stream_pkg.ALL;


ENTITY mms_dp_throttle IS
  GENERIC (
    g_dc_period      : NATURAL := 100;
    g_throttle_valid : BOOLEAN := FALSE 
  );
  PORT (
    mm_rst   : IN  STD_LOGIC; 
    mm_clk   : IN  STD_LOGIC; 

    dp_rst   : IN  STD_LOGIC; 
    dp_clk   : IN  STD_LOGIC; 

    reg_mosi : IN  t_mem_mosi;  -- actual ranges defined by c_mm_reg
    reg_miso : OUT t_mem_miso;  -- actual ranges defined by c_mm_reg
    
    snk_out  : OUT t_dp_siso;
    snk_in   : IN  t_dp_sosi;

    src_in   : IN  t_dp_siso;
    src_out  : OUT t_dp_sosi
  );
END mms_dp_throttle;


ARCHITECTURE str OF mms_dp_throttle IS

  SIGNAL throttle : STD_LOGIC_VECTOR(ceil_log2(g_dc_period+1)-1 DOWNTO 0);

BEGIN

  u_dp_throttle_reg : ENTITY work.dp_throttle_reg
  GENERIC MAP (
    g_dc_period => g_dc_period
  )
  PORT MAP (

    mm_rst   => mm_rst,
    mm_clk   => mm_clk,

    st_rst   => dp_rst,
    st_clk   => dp_clk,

    sla_in   => reg_mosi,
    sla_out  => reg_miso,

    throttle => throttle
  );
  
  u_dp_throttle : ENTITY work.dp_throttle
  GENERIC MAP (
    g_dc_period      => g_dc_period,
    g_throttle_valid => g_throttle_valid
  )
  PORT MAP (
    rst       => dp_rst,
    clk       => dp_clk,

    snk_out   => snk_out,
    snk_in    => snk_in,
    
    src_in    => src_in,
    src_out   => src_out,

    throttle  => throttle
  );
  
END str;

